@@ -2,15 +2,23 @@
 """
 The classification module provides classification functionality within RSGISLib.
 """
+# import the C++ extension into this level
+from ._classification import *
 
+import os
 from typing import Dict, List
 
 import rsgislib
 import rsgislib.imageutils
 from rsgislib.imageutils import ImageBandInfo
 
-# import the C++ extension into this level
-from ._classification import *
+TQDM_AVAIL = True
+try:
+    import tqdm
+except ImportError:
+    import rios.cuiprogress
+
+    TQDM_AVAIL = False
 
 
 class ClassSimpleInfoObj(object):
@@ -242,8 +250,6 @@ def get_class_info_dict(
              instance as the value.
 
     """
-    import os
-
     import rsgislib.tools.filetools
 
     cls_smpls_fnl_info = dict()
@@ -300,7 +306,6 @@ def get_class_training_data(
     :return: dictionary of ClassSimpleInfoObj objects.
 
     """
-    import os
     import random
     import shutil
 
@@ -394,7 +399,6 @@ def get_class_training_chips_data(
     :return: dictionary of ClassSimpleInfoObj objects.
 
     """
-    import os
     import random
     import shutil
 
@@ -595,8 +599,6 @@ def split_sample_train_valid_test(
                      then the output data type will be float32.
 
     """
-    import os
-
     import rsgislib.tools.filetools
     import rsgislib.tools.utils
     import rsgislib.zonalstats
@@ -681,8 +683,6 @@ def split_chip_sample_train_valid_test(
                      then the output data type will be float32.
 
     """
-    import os
-
     import rsgislib.tools.filetools
     import rsgislib.tools.utils
     import rsgislib.zonalstats
@@ -768,8 +768,6 @@ def split_chip_sample_ref_train_valid_test(
                      then the output data type will be float32.
 
     """
-    import os
-
     import rsgislib
     import rsgislib.tools.filetools
     import rsgislib.tools.utils
@@ -1139,11 +1137,8 @@ def plot_train_data(
                       otherwise call 'Var #1', 'Var #2' ... 'Var #N' etc.
 
     """
-    import os
-
     import h5py
     import numpy
-    import pandas
     import plotly.express as px
     import tqdm
 
@@ -1200,8 +1195,6 @@ def plot_train_data(
         df_data[var_names[var_n]] = cls_data[..., var_n]
     df_data["ClassName"] = cls_data_name
 
-    df = pandas.DataFrame(df_data)
-
     for var1 in tqdm.tqdm(var_names):
         for var2 in var_names:
             out_title = "{} verses {}".format(var1, var2)
@@ -1373,7 +1366,7 @@ def fill_class_timeseries(
                           for both the input and output images.
 
     """
-    from rios import applier, cuiprogress
+    from rios import applier
 
     import rsgislib.imageutils
     import rsgislib.rastergis
@@ -1413,12 +1406,10 @@ def fill_class_timeseries(
     other_args.n_imgs = len(input_imgs)
     other_args.n_iters = n_iters
 
-    try:
-        import tqdm
-
+    if TQDM_AVAIL:
         progress_bar = rsgislib.TQDMProgressBar()
-    except:
-        progress_bar = cuiprogress.GDALProgressBar()
+    else:
+        progress_bar = rios.cuiprogress.GDALProgressBar()
 
     a_controls = applier.ApplierControls()
     a_controls.progress = progress_bar
