@@ -3,6 +3,8 @@
 The tools.stats module contains some useful tools for calculating useful statistics which aren't
 easily available else where.
 """
+from typing import Union
+
 import numpy
 
 import rsgislib
@@ -1074,3 +1076,187 @@ def decompose_bias_variance(y_true, y_pred):
         if noise < 0:
             noise = 0
     return mse, bias_squared, variance, noise
+
+
+def calc_semivariogram(
+    pts_coords: numpy.array,
+    data_vals: numpy.array,
+    out_data_file: str = None,
+    out_plot_file: str = None,
+    max_lag: Union[float, str] = "median",
+    n_lags: int = 25,
+    normalize_vals: bool = False,
+):
+    """
+    A function which calculates semi-variance for the data provided using the
+    skgstat module (https://scikit-gstat.readthedocs.io/)
+
+    :param pts_coords: the x/y coordinates of the points for which the semi-variance is
+                       calculated. Shape must be [n, 2] where n is the number of points
+    :param data_vals: the data values of the points for which the semi-variance is
+                      calculated.
+    :param out_data_file: Optionally output a CSV file with the distance, semi-variance
+                          and count. Default is None but if file path provided the
+                          output will be produced.
+    :param out_plot_file: Optionally output a plot file of the distance, semi-variance
+                          and count. Default is None but if file path provided the
+                          output will be produced.
+    :param max_lag: Can specify the maximum lag distance directly by giving a value
+                    larger than 1. Can also be a string with value ‘median’, ‘mean’.
+                    See skgstat.Variogram documentation.
+    :param n_lags: Specify the number of lag classes to be defined by the binning
+                   function. See skgstat.Variogram documentation.
+    :param normalize_vals: Defaults to False. If True, the independent and dependent
+                           variable will be normalized to the range [0,1].
+                           See skgstat.Variogram documentation.
+    :return: returns a pandas dataframe with the lag_bins, variance and count
+
+    """
+    import matplotlib.pyplot as plt
+    import skgstat
+    import pandas
+
+    if pts_coords.shape[1] != 2:
+        raise rsgislib.RSGISPyException(
+            "Error: pts_coords has wrong shape. Must be [n, 2] "
+            "where n is the number of points."
+        )
+
+    if pts_coords.shape[0] != data_vals.shape[0]:
+        raise rsgislib.RSGISPyException(
+            "Error: the length of pts_coords must be the same as data_vals."
+        )
+
+    vario_obj = skgstat.Variogram(
+        pts_coords, data_vals, normalize=normalize_vals, maxlag=max_lag, n_lags=n_lags
+    )
+
+    variogram_data = vario_obj.get_empirical()
+
+    n_count = numpy.fromiter((g.size for g in vario_obj.lag_classes()), dtype=int)
+
+    vario_data = {
+        "distance": variogram_data[0],
+        "semivariance": variogram_data[1],
+        "count": n_count,
+    }
+
+    vario_out_df = pandas.DataFrame(vario_data)
+
+    if out_data_file is not None:
+        vario_out_df.to_csv(out_data_file)
+
+    if out_plot_file is not None:
+        fig = plt.figure(figsize=(8, 5))
+        ax1 = plt.subplot2grid((5, 1), (1, 0), rowspan=4)
+        ax2 = plt.subplot2grid((5, 1), (0, 0), sharex=ax1)
+        fig.subplots_adjust(hspace=0)
+
+        ax1.scatter(x=vario_out_df["distance"], y=vario_out_df["semivariance"])
+        ax1.set_xlabel("Distance")
+        ax1.set_ylabel("Semi-Variance")
+
+        if normalize_vals:
+            ax1.set_xlim([0, 1.05])
+            ax1.set_ylim([0, 1.05])
+
+        ax1.grid(False)
+        ax1.vlines(
+            vario_out_df["distance"],
+            *ax1.axes.get_ybound(),
+            colors=(0.85, 0.85, 0.85),
+            linestyles="dashed"
+        )
+
+        # set the sum of hist bar widths to 70% of the x-axis space
+        w = (numpy.max(vario_out_df["distance"]) * 0.7) / len(vario_out_df["count"])
+
+        # plot bar chart with count of number of pairs
+        ax2.bar(
+            vario_out_df["distance"],
+            vario_out_df["count"],
+            width=w,
+            align="center",
+            color="red",
+        )
+
+        plt.setp(ax2.axes.get_xticklabels(), visible=False)
+        ax2.axes.set_yticks(ax2.axes.get_yticks()[1:])
+
+        ax2.grid(False)
+        ax2.vlines(
+            vario_out_df["distance"],
+            *ax2.axes.get_ybound(),
+            colors=(0.85, 0.85, 0.85),
+            linestyles="dashed"
+        )
+        ax2.axes.set_ylabel("N")
+
+        # Save figure
+        plt.savefig(out_plot_file)
+
+    return vario_out_df
+
+
+def standarise_img_data(img_data: numpy.array) -> numpy.array:
+    """
+    Standardise the input image (minus the mean and
+    divide by the standard deviation).
+
+    :param img_data: a numpy array with the shape [bands, height, width] or
+                     [bands, n_pxls].
+    :return: a numpy array with the standarised pixel values.
+
+    """
+    if len(img_data.shape) == 3:
+        img_bands, img_height, img_width = img_data.shape
+        n_img_pxls = img_height * img_width
+        img_data_reshp = numpy.reshape(img_data, (img_bands, img_height * img_width))
+    elif len(img_data.shape) == 2:
+        img_bands, n_img_pxls = img_data.shape
+        img_data_reshp = img_data
+    else:
+        raise rsgislib.RSGISPyException(
+            "Input array must have either 2 or 3 dimensions."
+        )
+
+    img_band_mean = numpy.mean(img_data_reshp, axis=1, keepdims=True)
+    img_band_cen = img_data_reshp - img_band_mean
+    img_band_var = numpy.sum(numpy.power(img_band_cen, 2), axis=1, keepdims=True) / (
+        n_img_pxls
+    )
+    img_band_std = numpy.sqrt(img_band_var)
+    std_img_data = img_band_cen / img_band_std
+    if len(img_data.shape) == 3:
+        std_img_data = numpy.reshape(std_img_data, (img_bands, img_height, img_width))
+
+    return std_img_data
+
+
+def normalise_img_data(img_data: numpy.array) -> numpy.array:
+    """
+    Normalise the input image (data - min)/range.
+
+    :param img_data: a numpy array with the shape [bands, height, width]
+    :return: a numpy array with the normalised pixel values.
+
+    """
+    if len(img_data.shape) == 3:
+        img_bands, img_height, img_width = img_data.shape
+        img_data_reshp = numpy.reshape(img_data, (img_bands, img_height * img_width))
+    elif len(img_data.shape) == 2:
+        img_bands, n_img_pxls = img_data.shape
+        img_data_reshp = img_data
+    else:
+        raise rsgislib.RSGISPyException(
+            "Input array must have either 2 or 3 dimensions."
+        )
+
+    max_value = numpy.max(img_data_reshp, axis=1, keepdims=True)
+    min_value = numpy.min(img_data_reshp, axis=1, keepdims=True)
+    diff_value = max_value - min_value
+    nm_img = (img_data_reshp - min_value) / diff_value
+    if len(img_data.shape) == 3:
+        nm_img = numpy.reshape(nm_img, (img_bands, img_height, img_width))
+
+    return nm_img
