@@ -37,7 +37,7 @@ namespace rsgis{namespace calib{
         this->useNoDataVal = useNoDataVal;
     }
     
-    void RSGISApply6SCoefficientsSingleParam::calcImageValue(float *bandValues, int numBands, double *output) throw(rsgis::img::RSGISImageCalcException)
+    void RSGISApply6SCoefficientsSingleParam::calcImageValue(float *bandValues, int numBands, double *output) 
     {
         if(numValues != this->numOutBands)
         {
@@ -101,6 +101,10 @@ namespace rsgis{namespace calib{
                         output[i] = output[i] + 1.0;
                     }
                 }
+                if(output[i] > this->scaleFactor)
+                {
+                    output[i] = this->scaleFactor;
+                }
             }
         }
         
@@ -135,7 +139,7 @@ namespace rsgis{namespace calib{
         }
     }
     
-    void RSGISApply6SCoefficientsElevLUTParam::calcImageValue(float *bandValues, int numBands, double *output) throw(rsgis::img::RSGISImageCalcException)
+    void RSGISApply6SCoefficientsElevLUTParam::calcImageValue(float *bandValues, int numBands, double *output) 
     {
         if(numBands-1 != this->numOutBands)
         {
@@ -278,6 +282,10 @@ namespace rsgis{namespace calib{
                         output[i] = output[i] + 1.0;
                     }
                 }
+                if(output[i] > this->scaleFactor)
+                {
+                    output[i] = this->scaleFactor;
+                }
             }
         }
         
@@ -299,7 +307,7 @@ namespace rsgis{namespace calib{
         this->useNoDataVal = useNoDataVal;
     }
     
-    void RSGISApply6SCoefficientsElevAOTLUTParam::calcImageValue(float *bandValues, int numBands, double *output) throw(rsgis::img::RSGISImageCalcException)
+    void RSGISApply6SCoefficientsElevAOTLUTParam::calcImageValue(float *bandValues, int numBands, double *output) 
     {
         if(numBands-2 != this->numOutBands)
         {
@@ -420,6 +428,10 @@ namespace rsgis{namespace calib{
                         output[i] = output[i] + 1.0;
                     }
                 }
+                if(output[i] > this->scaleFactor)
+                {
+                    output[i] = this->scaleFactor;
+                }
             }
         }
         
@@ -452,7 +464,7 @@ namespace rsgis{namespace calib{
 		}
     }
     
-    void RSGISApply6SCoefficients::calcImageValue(float *bandValues, int numBands, double *output) throw(rsgis::img::RSGISImageCalcException)
+    void RSGISApply6SCoefficients::calcImageValue(float *bandValues, int numBands, double *output) 
     {
         if(numValues != this->numOutBands)
         {
@@ -530,7 +542,7 @@ namespace rsgis{namespace calib{
         this->solarZenith = solarZenith;
     }
     
-    void RSGISCalcSolarIrradianceElevLUTParam::calcImageValue(float *bandValues, int numBands, double *output) throw(rsgis::img::RSGISImageCalcException)
+    void RSGISCalcSolarIrradianceElevLUTParam::calcImageValue(float *bandValues, int numBands, double *output) 
     {
         /*
          * bandValues[0] - valid data mask
@@ -713,7 +725,7 @@ namespace rsgis{namespace calib{
         this->reflScaleFactor = reflScaleFactor;
     }
     
-    void RSGISCalcStandardisedReflectanceSD2010::calcImageValue(float *bandValues, int numBands, double *output) throw(rsgis::img::RSGISImageCalcException)
+    void RSGISCalcStandardisedReflectanceSD2010::calcImageValue(float *bandValues, int numBands, double *output) 
     {
         /*
          * bandValues[0] - valid data mask
@@ -758,6 +770,7 @@ namespace rsgis{namespace calib{
                 
                 LVal = (bandValues[srefBandIdx] / this->reflScaleFactor) * bandValues[irrTotalBandIdx];
                 // I am not multipling by Pi (Eq 8) as I have not divided by Pui (Eq 4).
+                // Checked *gamma rather than /gamma (Eq 8). *gamma corrects slope /gamma increases topo refl
                 output[i] = (LVal) / ((bandValues[irrDirBandIdx] * gamma) + (bandValues[irrDifandIdx] * this->brdfBeta));
                 
                 output[i] = output[i] * reflScaleFactor;
