@@ -30,47 +30,181 @@
 #include "registration/RSGISImageSimilarityMetric.h"
 #include "registration/RSGISStandardImageSimilarityMetrics.h"
 #include "registration/RSGISSingleConnectLayerImageRegistration.h"
-#include "registration/RSGISWarpImage.h"
-#include "registration/RSGISBasicNNGCPImageWarp.h"
-#include "registration/RSGISWarpImageInterpolator.h"
-#include "registration/RSGISWarpImageUsingTriangulation.h"
-#include "registration/RSGISPolynomialImageWarp.h"
 #include "registration/RSGISAddGCPsGDAL.h"
+#include "registration/RSGISFindImageOffset.h"
+
 
 #include "img/RSGISCalcImageValue.h"
 #include "img/RSGISCalcImage.h"
 #include "img/RSGISCopyImage.h"
 
+#include "utils/RSGISTextUtils.h"
+
+
 
 namespace rsgis{ namespace cmds {
 
+    std::pair<double, double> excecuteFindImageOffset(std::string inputReferenceImage, std::string inputFloatingmage,
+                                                      std::vector<unsigned int> refImageBands,
+                                                      std::vector<unsigned int> fltImageBands,
+                                                      unsigned int xSearch, unsigned int ySearch,
+                                                      unsigned int metricTypeInt,
+                                                      int subPixelResolution)
+    {
+        std::pair<double, double> imgOffsets;
+        try
+        {
+            rsgis::utils::RSGISTextUtils txtUtils;
+            GDALAllRegister();
+            GDALDataset *inRefDataset = nullptr;
+            GDALDataset *inFloatDataset = nullptr;
+
+            inRefDataset = (GDALDataset *) GDALOpenShared(inputReferenceImage.c_str(), GA_ReadOnly);
+            if(inRefDataset == nullptr)
+            {
+                std::string message = std::string("Could not open image ") + inputReferenceImage;
+                throw rsgis::RSGISException(message.c_str());
+            }
+            int nRefBands = inRefDataset->GetRasterCount();
+
+            inFloatDataset = (GDALDataset *) GDALOpenShared(inputFloatingmage.c_str(), GA_ReadOnly);
+            if(inFloatDataset == nullptr)
+            {
+                std::string message = std::string("Could not open image ") + inputFloatingmage;
+                throw rsgis::RSGISException(message.c_str());
+            }
+            int nFltBands = inFloatDataset->GetRasterCount();
+
+            if(refImageBands.size() != fltImageBands.size())
+            {
+                throw rsgis::RSGISException("The number of bands specified from the reference and floating images must be the same.");
+            }
+
+            std::vector<unsigned int> refImgBandsArrIdx;
+            std::vector<unsigned int> fltImgBandsArrIdx;
+
+            double refImgNoData = 0;
+            int useRefImgNoData = false;
+            double fltImgNoData = 0;
+            int useFltImgNoData = false;
+
+            bool first = true;
+            for(auto iterBand = refImageBands.begin(); iterBand != refImageBands.end(); ++iterBand)
+            {
+                if((*iterBand) < 1)
+                {
+                    throw rsgis::RSGISException("Image band indexing starts at 1");
+                }
+                if((*iterBand) > nRefBands)
+                {
+                    std::string message = std::string("Image band ") + txtUtils.uInttostring((*iterBand)) + std::string(" is not within the reference image.");
+                    throw rsgis::RSGISException(message.c_str());
+                }
+                refImgBandsArrIdx.push_back((*iterBand)-1);
+                if(first)
+                {
+                    refImgNoData = inRefDataset->GetRasterBand((*iterBand))->GetNoDataValue(&useRefImgNoData);
+                    first = false;
+                }
+            }
+
+            first = true;
+            for(auto iterBand = fltImageBands.begin(); iterBand != fltImageBands.end(); ++iterBand)
+            {
+                if((*iterBand) < 1)
+                {
+                    throw rsgis::RSGISException("Image band indexing starts at 1");
+                }
+                if((*iterBand) > nFltBands)
+                {
+                    std::string message = std::string("Image band ") + txtUtils.uInttostring((*iterBand)) + std::string(" is not within the floating image.");
+                    throw rsgis::RSGISException(message.c_str());
+                }
+                fltImgBandsArrIdx.push_back(nRefBands+((*iterBand)-1));
+                if(first)
+                {
+                    fltImgNoData = inRefDataset->GetRasterBand((*iterBand))->GetNoDataValue(&useFltImgNoData);
+                    first = false;
+                }
+            }
+
+            rsgis::reg::RSGISImageCalcSimilarityMetric *similarityMetric = nullptr;
+            if(metricTypeInt == 1) // euclidean
+            {
+                similarityMetric = new rsgis::reg::RSGISImgCalcEuclideanSimilarityMetric(refImgBandsArrIdx, fltImgBandsArrIdx, refImgNoData, useRefImgNoData, fltImgNoData, useFltImgNoData);
+            }
+            else if(metricTypeInt == 2) // sqdiff
+            {
+                similarityMetric = new rsgis::reg::RSGISImgCalcSquaredDifferenceSimilarityMetric(refImgBandsArrIdx, fltImgBandsArrIdx, refImgNoData, useRefImgNoData, fltImgNoData, useFltImgNoData);
+            }
+            else if(metricTypeInt == 3) // manhatten
+            {
+                similarityMetric = new rsgis::reg::RSGISImgCalcManhattanSimilarityMetric(refImgBandsArrIdx, fltImgBandsArrIdx, refImgNoData, useRefImgNoData, fltImgNoData, useFltImgNoData);
+            }
+            else if(metricTypeInt == 4) // correlation
+            {
+                similarityMetric = new rsgis::reg::RSGISImgCalcCorrelationSimilarityMetric(refImgBandsArrIdx, fltImgBandsArrIdx, refImgNoData, useRefImgNoData, fltImgNoData, useFltImgNoData);
+            }
+            else
+            {
+                throw rsgis::cmds::RSGISCmdException("Metric not recognised!");
+            }
+            std::cout << "subPixelResolution = " << subPixelResolution << std::endl;
+            bool calcSubPxl = false;
+            if(subPixelResolution > 0)
+            {
+                calcSubPxl = true;
+            }
+
+            // DO ANALYSIS!!
+            rsgis::reg::RSGISFindImageOffset findImageOffset;
+            imgOffsets = findImageOffset.findImageOffset(inRefDataset, inFloatDataset,
+                                                                                   xSearch, ySearch,
+                                                                                   similarityMetric,
+                                                                                   calcSubPxl, subPixelResolution);
+
+            GDALClose(inRefDataset);
+            GDALClose(inFloatDataset);
+        }
+        catch(RSGISException& e)
+        {
+            throw RSGISCmdException(e.what());
+        }
+        catch(std::exception& e)
+        {
+            throw RSGISCmdException(e.what());
+        }
+
+        return imgOffsets;
+    }
+
     void excecuteBasicRegistration(std::string inputReferenceImage, std::string inputFloatingmage, int gcpGap,
                                                   float metricThreshold, int windowSize, int searchArea, float stdDevRefThreshold,
                                                   float stdDevFloatThreshold, int subPixelResolution, unsigned int metricTypeInt,
-                                                  unsigned int outputType, std::string outputGCPFile) throw(RSGISCmdException)
+                                                  unsigned int outputType, std::string outputGCPFile) 
     {
         
         try
         {
             GDALAllRegister();
-            GDALDataset *inRefDataset = NULL;
-            GDALDataset *inFloatDataset = NULL;
+            GDALDataset *inRefDataset = nullptr;
+            GDALDataset *inFloatDataset = nullptr;
             
             inRefDataset = (GDALDataset *) GDALOpenShared(inputReferenceImage.c_str(), GA_ReadOnly);
-            if(inRefDataset == NULL)
+            if(inRefDataset == nullptr)
             {
                 std::string message = std::string("Could not open image ") + inputReferenceImage;
                 throw rsgis::RSGISException(message.c_str());
             }
             
             inFloatDataset = (GDALDataset *) GDALOpenShared(inputFloatingmage.c_str(), GA_ReadOnly);
-            if(inFloatDataset == NULL)
+            if(inFloatDataset == nullptr)
             {
                 std::string message = std::string("Could not open image ") + inputFloatingmage;
                 throw rsgis::RSGISException(message.c_str());
             }
             
-            rsgis::reg::RSGISImageSimilarityMetric *similarityMetric = NULL;
+            rsgis::reg::RSGISImageSimilarityMetric *similarityMetric = nullptr;
             if(metricTypeInt == 1) // euclidean
             {
                 similarityMetric = new rsgis::reg::RSGISEuclideanSimilarityMetric();
@@ -135,30 +269,30 @@ namespace rsgis{ namespace cmds {
                                                   float metricThreshold, int windowSize, int searchArea, float stdDevRefThreshold,
                                                   float stdDevFloatThreshold, int subPixelResolution, int distanceThreshold,
                                                   int maxNumIterations, float moveChangeThreshold, float pSmoothness, unsigned int metricTypeInt,
-                                                  unsigned int outputType, std::string outputGCPFile) throw(RSGISCmdException)
+                                                  unsigned int outputType, std::string outputGCPFile) 
     {
                 
         try
         {
             GDALAllRegister();
-            GDALDataset *inRefDataset = NULL;
-            GDALDataset *inFloatDataset = NULL;
+            GDALDataset *inRefDataset = nullptr;
+            GDALDataset *inFloatDataset = nullptr;
             
             inRefDataset = (GDALDataset *) GDALOpenShared(inputReferenceImage.c_str(), GA_ReadOnly);
-            if(inRefDataset == NULL)
+            if(inRefDataset == nullptr)
             {
                 std::string message = std::string("Could not open image ") + inputReferenceImage;
                 throw rsgis::RSGISException(message.c_str());
             }
             
             inFloatDataset = (GDALDataset *) GDALOpenShared(inputFloatingmage.c_str(), GA_ReadOnly);
-            if(inFloatDataset == NULL)
+            if(inFloatDataset == nullptr)
             {
                 std::string message = std::string("Could not open image ") + inputFloatingmage;
                 throw rsgis::RSGISException(message.c_str());
             }
             
-            rsgis::reg::RSGISImageSimilarityMetric *similarityMetric = NULL;
+            rsgis::reg::RSGISImageSimilarityMetric *similarityMetric = nullptr;
             if(metricTypeInt == 1) // euclidean
             {
                 similarityMetric = new rsgis::reg::RSGISEuclideanSimilarityMetric();
@@ -221,120 +355,9 @@ namespace rsgis{ namespace cmds {
             throw RSGISCmdException(e.what());
         }
     }
+
     
-    void excecuteNNWarp(std::string inputImage, std::string outputImage, std::string projFile, std::string inputGCPs, float resolution, std::string imageFormat, bool genTransformImage) throw(RSGISCmdException)
-    {
-        
-        try
-        {
-            GDALAllRegister();
-            rsgis::reg::RSGISWarpImage *warp = NULL;
-            rsgis::reg::RSGISWarpImageInterpolator *interpolator = new rsgis::reg::RSGISWarpImageNNInterpolator();
-            
-            std::string projWKTStr = "";
-            if(projFile != "")
-            {
-                rsgis::utils::RSGISTextUtils textUtils;
-                projWKTStr = textUtils.readFileToString(projFile);
-            }
-            
-            warp = new rsgis::reg::RSGISBasicNNGCPImageWarp(inputImage, outputImage, projWKTStr, inputGCPs, resolution, interpolator, imageFormat);
-            if(genTransformImage)
-            {
-                warp->generateTransformImage();
-            }
-            else
-            {
-                warp->performWarp();
-            }
-            delete warp;
-        }
-        catch(RSGISException& e)
-        {
-            throw RSGISCmdException(e.what());
-        }
-        catch(std::exception& e)
-        {
-            throw RSGISCmdException(e.what());
-        }
-    }
-    
-    void excecuteTriangularWarp(std::string inputImage, std::string outputImage, std::string projFile, std::string inputGCPs, float resolution, std::string imageFormat, bool genTransformImage) throw(RSGISCmdException)
-    {
-        
-        try
-        {
-            GDALAllRegister();
-            rsgis::reg::RSGISWarpImage *warp = NULL;
-            rsgis::reg::RSGISWarpImageInterpolator *interpolator = new rsgis::reg::RSGISWarpImageNNInterpolator();
-            
-            std::string projWKTStr = "";
-            if(projFile != "")
-            {
-                rsgis::utils::RSGISTextUtils textUtils;
-                projWKTStr = textUtils.readFileToString(projFile);
-            }
-            
-            warp = new rsgis::reg::RSGISWarpImageUsingTriangulation(inputImage, outputImage, projWKTStr, inputGCPs, resolution, interpolator, imageFormat);
-            if(genTransformImage)
-            {
-                warp->generateTransformImage();
-            }
-            else
-            {
-                warp->performWarp();
-            }
-            delete warp;
-        }
-        catch(RSGISException& e)
-        {
-            throw RSGISCmdException(e.what());
-        }
-        catch(std::exception& e)
-        {
-            throw RSGISCmdException(e.what());
-        }
-    }
-    
-    
-    void excecutePolyWarp(std::string inputImage, std::string outputImage, std::string projFile, std::string inputGCPs, float resolution, int polyOrder, std::string imageFormat, bool genTransformImage) throw(RSGISCmdException)
-    {
-        
-        try
-        {
-            GDALAllRegister();
-            rsgis::reg::RSGISWarpImage *warp = NULL;
-            rsgis::reg::RSGISWarpImageInterpolator *interpolator = new rsgis::reg::RSGISWarpImageNNInterpolator();
-            
-            std::string projWKTStr = "";
-            if(projFile != "")
-            {
-                rsgis::utils::RSGISTextUtils textUtils;
-                projWKTStr = textUtils.readFileToString(projFile);
-            }
-            
-            warp = new rsgis::reg::RSGISPolynomialImageWarp(inputImage, outputImage, projWKTStr, inputGCPs, resolution, interpolator, polyOrder, imageFormat);
-            if(genTransformImage)
-            {
-                warp->generateTransformImage();
-            }
-            else
-            {
-                warp->performWarp();
-            }
-            delete warp;
-        }
-        catch(RSGISException& e)
-        {
-            throw RSGISCmdException(e.what());
-        }
-        catch(std::exception& e)
-        {
-            throw RSGISCmdException(e.what());
-        }
-    }
-    
-    void excecuteAddGCPsGDAL(std::string inputImage, std::string inputGCPs, std::string outputImage, std::string gdalFormat, RSGISLibDataType outDataType) throw(RSGISCmdException)
+    void excecuteAddGCPsGDAL(std::string inputImage, std::string inputGCPs, std::string outputImage, std::string gdalFormat, RSGISLibDataType outDataType) 
     {
         try
         {
@@ -353,7 +376,7 @@ namespace rsgis{ namespace cmds {
     }
     
     
-    void executeApplyOffset2Image(std::string inputImage, std::string outputImage, std::string gdalFormat, RSGISLibDataType outDataType, double xOff, double yOff) throw(RSGISCmdException)
+    void executeApplyOffset2Image(std::string inputImage, std::string outputImage, std::string gdalFormat, RSGISLibDataType outDataType, double xOff, double yOff) 
     {
         try
         {
@@ -361,7 +384,7 @@ namespace rsgis{ namespace cmds {
             
             std::cout << inputImage << std::endl;
             GDALDataset *dataset = (GDALDataset *) GDALOpen(inputImage.c_str(), GA_ReadOnly);
-            if(dataset == NULL)
+            if(dataset == nullptr)
             {
                 std::string message = std::string("Could not open image ") + inputImage;
                 throw rsgis::RSGISImageException(message.c_str());
@@ -371,13 +394,13 @@ namespace rsgis{ namespace cmds {
             
             rsgis::img::RSGISCopyImage copyImage = rsgis::img::RSGISCopyImage(numBands);
             rsgis::img::RSGISCalcImage *calcImage = new rsgis::img::RSGISCalcImage(&copyImage, "", true);
-            calcImage->calcImage(&dataset, 1, outputImage, false, NULL, gdalFormat, RSGIS_to_GDAL_Type(outDataType));
+            calcImage->calcImage(&dataset, 1, outputImage, false, nullptr, gdalFormat, RSGIS_to_GDAL_Type(outDataType));
             
             delete calcImage;
             GDALClose(dataset);
             
             GDALDataset *outDataset = (GDALDataset *) GDALOpen(outputImage.c_str(), GA_Update);
-            if(outDataset == NULL)
+            if(outDataset == nullptr)
             {
                 std::string message = std::string("Could not open image ") + outputImage;
                 throw rsgis::RSGISImageException(message.c_str());
